# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Gustavo Noronha Silva
# Copyright (©) 2013 Marcelo Jorge Vieira <metal@alucinados.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import patterns, include, url
from django.conf.urls.static import static
from django.conf import settings

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    #Admin
    url(r'^admin', include(admin.site.urls)),

    # Montanha
    url(r'^', include('montanha.urls',
                      namespace='montanha',
                      app_name='montanha')),
)

urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

handler500 = 'montanha.views.error_500'
handler404 = 'montanha.views.error_404'



# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Estêvão Samuel Procópio
# Copyright (©) 2010-2013 Gustavo Noronha Silva
# Copyright (©) 2014 Lúcio Flávio Corrêa
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from datetime import datetime
from StringIO import StringIO
from django.db import reset_queries
import pandas as pd
from basecollector import BaseCollector
from montanha.models import (ArchivedExpense, Institution, Legislature,
                             Legislator, Mandate, ExpenseNature, Supplier, PoliticalParty)

OBJECT_LIST_MAXIMUM_COUNTER = 1000


class Senado(BaseCollector):
    def __init__(self, collection_runs, debug_enabled=False):
        super(Senado, self).__init__(collection_runs, debug_enabled)

        institution, _ = Institution.objects.get_or_create(siglum='Senado', name=u'Senado Federal')
        self.legislature, _ = Legislature.objects.get_or_create(institution=institution,
                                                                date_start=datetime(2015, 1, 1),
                                                                date_end=datetime(2018, 12, 31))

    def retrieve_legislators(self):
        uri = 'http://www25.senado.leg.br/web/transparencia/sen/'
        return BaseCollector.retrieve_uri(self, uri, force_encoding='utf-8')

    def retrieve_data_for_year(self, year):
        uri = 'http://www.senado.gov.br/transparencia/LAI/verba/%d.csv' % year

        self.debug("Downloading %s" % uri)

        return BaseCollector.retrieve_uri(self, uri, force_encoding='windows-1252', post_process=False)

    def try_name_disambiguation(self, name):
        if name.title() == 'Luiz Henrique':
            return Legislator.objects.get(id=246), False

        return None, False

    def update_legislators(self):
        page = self.retrieve_legislators()

        # We ignore the first one because it is a placeholder.
        options = page(attrs={'name': 'frmConsulta1'})[0].findAll('option')[1:]

        # Turn the soup objects into a list of dictionaries
        legislators = []
        for item in options:
            name = ' '.join([x.title() for x in item.getText().split()])
            original_id = int(item.get('value'))
            legislators.append(dict(name=name, original_id=original_id))

        # Add legislators that do not exist yet
        for l in legislators:
            mandate = Mandate.objects.filter(legislature=self.legislature).filter(original_id=l['original_id'])
            if mandate.count():
                assert(mandate.count() == 1)
                mandate = mandate[0]
                legislator = mandate.legislator
                self.debug("Found existing legislator: %s" % unicode(legislator))
            else:
                legislator, created = self.try_name_disambiguation(l['name'])
                if not legislator:
                    legislator, created = Legislator.objects.get_or_create(name=l['name'])

                if created:
                    self.debug("New legislator: %s" % unicode(legislator))
                else:
                    self.debug("Found existing legislator: %s" % unicode(legislator))

                mandate = self.mandate_for_legislator(legislator, party=None, original_id=l['original_id'])

    def update_data(self):
        self.collection_run = self.create_collection_run(self.legislature)
        for year in range(self.legislature.date_start.year, datetime.now().year + 1):
            self.update_data_for_year(year)

    def update_data_for_year(self, year=datetime.now().year):
        self.debug("Updating data for year %d" % year)

        csv_data = self.retrieve_data_for_year(year)
        csv_data = re.sub(r'([^;])"([^;\n])', r'\1\"\2', csv_data)
        data = StringIO(csv_data)

        if data:
            df = pd.read_csv(data, skiprows=1, delimiter=";",
                             parse_dates=[7], decimal=',',
                             error_bad_lines=False,
                             encoding='utf-8').dropna(how='all')

            expected_header = [u'ANO',
                               u'MES',
                               u'SENADOR',
                               u'TIPO_DESPESA',
                               u'CNPJ_CPF',
                               u'FORNECEDOR',
                               u'DOCUMENTO',
                               u'DATA',
                               u'DETALHAMENTO',
                               u'VALOR_REEMBOLSADO']

            actual_header = df.columns.values.tolist()

            if actual_header != expected_header:
                print u'Bad CSV: expected header %s, got %s' % (expected_header, actual_header)
                return

            archived_expense_list = []
            objects_counter = 0
            archived_expense_list_counter = len(df.index)

            for idx, row in df.iterrows():
                name = row["SENADOR"]
                nature = row["TIPO_DESPESA"]
                cpf_cnpj = self.normalize_cnpj_or_cpf(row["CNPJ_CPF"])
                supplier_name = row["FORNECEDOR"]
                docnumber = row["DOCUMENTO"]
                expense_date = row["DATA"]
                expensed = row['VALOR_REEMBOLSADO']

                # FIXME: WTF?
                if isinstance(expensed, unicode):
                    expensed = float(expensed.replace(',', '.').replace('\r\n', ''))

                nature, _ = ExpenseNature.objects.get_or_create(name=nature)

                try:
                    supplier = Supplier.objects.get(identifier=cpf_cnpj)
                except Supplier.DoesNotExist:
                    supplier = Supplier(identifier=cpf_cnpj, name=supplier_name)
                    supplier.save()

                legislator, _ = self.try_name_disambiguation(name)
                if not legislator:
                    legislator = Legislator.objects.get(name__iexact=name)
                mandate = self.mandate_for_legislator(legislator, None)
                expense = ArchivedExpense(number=docnumber,
                                          nature=nature,
                                          date=expense_date,
                                          expensed=expensed,
                                          mandate=mandate,
                                          supplier=supplier,
                                          collection_run=self.collection_run)
                archived_expense_list.append(expense)
                self.debug("New expense found: %s" % unicode(expense))

                objects_counter += 1
                archived_expense_list_counter -= 1

                # We create a list with up to OBJECT_LIST_MAXIMUM_COUNTER.
                # If that lists is equal to the maximum object count allowed
                # or if there are no more objects in archived_expense_list,
                # we bulk_create() them and clear the list.

                if objects_counter == OBJECT_LIST_MAXIMUM_COUNTER or archived_expense_list_counter == 0:
                    ArchivedExpense.objects.bulk_create(archived_expense_list)
                    archived_expense_list[:] = []
                    objects_counter = 0
                    reset_queries()
        else:
            self.debug("Error downloading file for year %d" % year)

    def _normalize_name(self, name):
        names_map = {
            'Gim': 'Gim Argello',
        }
        return names_map.get(name, name)

    def _normalize_party_name(self, name):
        names_map = {
            'PCdoB': 'PC do B',
        }
        return names_map.get(name, name)

    def update_legislators_extra_data(self):
        # Disabled for now, page got redesigned
        return
        data = self.retrieve_uri('http://www.senado.gov.br/senadores/')
        table = data.find(id='senadores')
        for row in table.findAll('tr'):
            columns = row.findAll('td')
            if not columns:
                continue

            name = self._normalize_name(columns[0].getText())

            legislator = Legislator.objects.filter(name=name).filter(
                mandate__legislature=self.legislature
            ).order_by("-mandate__date_start")

            # Check if query returned a legislator
            if legislator.exists():
                # Get the first row of the query result
                legislator = legislator[0]

                mandate = legislator.mandate_set.order_by("-date_start")[0]
                if mandate.legislature != self.legislature:
                    print 'Legislature found for %s is not the same as the one we need, ignoring.' % legislator.name
                    continue

                party = self._normalize_party_name(columns[1].getText())
                mandate.party, _ = PoliticalParty.objects.get_or_create(siglum=party)
                mandate.save()

                href = columns[6].findChild().get('href')
                if href:
                    legislator.email = href.split(':')[1]

                href = columns[7].findChild().get('href')
                if href:
                    legislator.site = href.split(':')[1]

                legislator.save()

                self.debug('Updated data for %s: %s, %s, %s' % (legislator.name,
                                                                mandate.party.name,
                                                                legislator.email,
                                                                legislator.site))
            else:
                self.debug('Legislator found on site but not on database: %s' % name)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Estêvão Samuel Procópio
# Copyright (©) 2010-2013 Gustavo Noronha Silva
# Copyright (©) 2014 Lúcio Flávio Corrêa
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, date
import time
import requests
from BeautifulSoup import BeautifulSoup, BeautifulStoneSoup
from montanha.models import Mandate, CollectionRun, ArchivedExpense


class BaseCollector(object):
    def __init__(self, collection_runs, debug_enabled):
        self.debug_enabled = debug_enabled
        self.collection_runs = collection_runs
        self.collection_run = None

        self.default_timeout = 20
        self.max_tries = 10
        self.try_again_timer = 10

    def debug(self, message):
        message = message.encode('utf-8')

        if self.debug_enabled:
            print message

        if not hasattr(self, 'logfile'):
            self.logfile = open(self.__class__.__name__.lower() + '.log', 'a')

        timestamp = datetime.fromtimestamp(time.time()).strftime('%F:%H:%M:%S')
        self.logfile.write('%s %s\n' % (timestamp, message))

    def mandate_for_legislator(self, legislator, party, state=None, original_id=None):
        try:
            mandate = Mandate.objects.get(legislator=legislator, date_start=self.legislature.date_start)
        except Mandate.DoesNotExist:
            mandate = Mandate(legislator=legislator, date_start=self.legislature.date_start, party=party,
                              legislature=self.legislature)
            mandate.save()
            self.debug("Mandate starting on %s did not exist, created." % self.legislature.date_start.strftime("%F"))

        if original_id:
            mandate.original_id = original_id
            mandate.save()

        return mandate

    def update_legislators(self):
        raise Exception("Not implemented.")

    def create_collection_run(self, legislature):
        collection_run, created = CollectionRun.objects.get_or_create(date=date.today(),
                                                                      legislature=legislature)
        self.collection_runs.append(collection_run)

        # Keep only one run for a day. If one exists, we delete the existing collection data
        # before we start this one.
        if not created:
            self.debug("Collection run for %s already exists for legislature %s, clearing." % (date.today().strftime("%F"), legislature))
            ArchivedExpense.objects.filter(collection_run=collection_run).delete()

        return collection_run

    def update_data(self):
        self.collection_run = self.create_collection_run(self.legislature)
        for mandate in Mandate.objects.filter(date_start__year=self.legislature.date_start.year,
                                              legislature=self.legislature):
            for year in range(self.legislature.date_start.year, datetime.now().year + 1):
                self.update_data_for_year(mandate, year)

    def retrieve_uri(self, uri, data=None, headers=None, post_process=True, force_encoding=None, return_content=False):
        retries = 0

        pargs = (uri, unicode(data), unicode(headers), int(post_process), unicode(force_encoding))
        self.debug(u"Retrieving %s data: %s headers: %s post_process? %d force_encoding: %s" % pargs)

        while retries < self.max_tries:
            try:
                options = dict(data=data, headers=headers, timeout=self.default_timeout, stream=True)
                if data:
                    r = requests.post(uri, **options)
                else:
                    r = requests.get(uri, **options)

                if force_encoding:
                    r.encoding = force_encoding
                if r.status_code == requests.codes.not_found:
                    return False
                if post_process:
                    return self.post_process_uri(r.text)
                elif return_content:
                    return r.content
                else:
                    return r.text

            except requests.ConnectionError:
                retries += 1
                print "Unable to retrieve %s try(%d) - will try again in %d seconds." % (uri, retries, self.try_again_timer)

            time.sleep(self.try_again_timer)

        raise RuntimeError("Error: Unable to retrieve %s; Tried %d times." % (uri, self.max_tries))

    def _normalize_party_name(self, name):
        names_map = {
            'PCdoB': 'PC do B',
        }
        return names_map.get(name, name)

    def post_process_uri(self, contents):
        return BeautifulSoup(contents, convertEntities=BeautifulStoneSoup.ALL_ENTITIES)

    def normalize_cnpj_or_cpf(self, cnpj):
        return cnpj.replace('.', '').replace('-', '').replace('/', '').strip()



# -*- coding: utf-8 -*-
#
# Copyright (©) 2013 Gustavo Noronha Silva
# Copyright (©) 2013 Marcelo Jorge Vieira
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from montanha.models import *


class HasWikipediaListFilter(admin.SimpleListFilter):
    title = _('Has wikipedia')
    parameter_name = 'wikipedia'

    def lookups(self, request, model_admin):
        return (
            ('yes', _('yes')),
            ('no', _('no')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'yes':
            return queryset.filter(wikipedia__isnull=False)
        if self.value() == 'no':
            return queryset.filter(wikipedia__isnull=True)


class PoliticalPartyAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'name', 'show_logo']
    ordering = ['siglum']
    search_fields = ['name']
    list_filter = [HasWikipediaListFilter]

    def show_logo(self, obj):
        if obj.logo:
            return mark_safe('<img src="%s" width="32" />' % obj.logo.url)
    show_logo.allow_tags = True
    show_logo.short_description = 'Logo'


class LegislatorAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'show_picture']
    ordering = ['name']
    search_fields = ['name']

    def show_picture(self, obj):
        if obj.picture:
            return mark_safe('<img src="%s" width="32" />' % obj.picture.url)
    show_picture.allow_tags = True
    show_picture.short_description = _('Picture')


admin.site.register(Legislator, LegislatorAdmin)
admin.site.register(Mandate)
admin.site.register(Supplier)
admin.site.register(ExpenseNature)
admin.site.register(Expense)
admin.site.register(PoliticalParty, PoliticalPartyAdmin)
admin.site.register(Institution)
admin.site.register(Legislature)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2014 Gustavo Noronha Silva
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import signal
import sys

# This hack makes django less memory hungry (it caches queries when running
# with debug enabled.
from django.conf import settings
settings.DEBUG = False

from django.core.management import call_command
from django.core.management.base import BaseCommand
from django.db import connection, transaction
from montanha.models import Expense


debug_enabled = False


class Command(BaseCommand):
    args = "<source> [debug]"
    help = "Collects data for a number of sources"

    def handle(self, *args, **options):
        global debug_enabled

        settings.expense_locked_for_collection = True

        collection_runs = []

        debug_enabled = False
        if "debug" in args:
            debug_enabled = True

        houses_to_consolidate = []

        if "almg" in args:
            from collectors.almg import ALMG
            almg = ALMG(collection_runs, debug_enabled)
            almg.update_legislators()
            almg.update_data()
            almg.update_legislators_data()
            houses_to_consolidate.append("almg")

        if "algo" in args:
            from collectors.algo import ALGO
            almg = ALGO(collection_runs, debug_enabled)
            almg.update_legislators()
            almg.update_data()
            houses_to_consolidate.append("algo")

        if "senado" in args:
            from collectors.senado import Senado
            senado = Senado(collection_runs, debug_enabled)
            senado.update_legislators()
            senado.update_data()
            senado.update_legislators_extra_data()
            houses_to_consolidate.append("senado")

        if "cmbh" in args:
            from collectors.cmbh import CMBH
            cmbh = CMBH(collection_runs, debug_enabled)
            cmbh.update_legislators()
            cmbh.update_data()
            houses_to_consolidate.append("cmbh")

        if "cmsp" in args:
            from collectors.cmsp import CMSP
            cmsp = CMSP(collection_runs, debug_enabled)
            cmsp.update_data()
            houses_to_consolidate.append("cmsp")

        if "cdep" in args:
            from collectors.cdep import CamaraDosDeputados
            cdep = CamaraDosDeputados(collection_runs, debug_enabled)
            cdep.update_legislators()
            cdep.update_data()
            houses_to_consolidate.append("cdep")

        settings.expense_locked_for_collection = False

        for run in collection_runs:
            legislature = run.legislature
            Expense.objects.filter(mandate__legislature=legislature).delete()

            columns = "number, nature_id, date, value, expensed, mandate_id, supplier_id"

            cursor = connection.cursor()
            cursor.execute("insert into montanha_expense (%s) select %s from montanha_archivedexpense where collection_run_id=%d" % (columns, columns, run.id))
            cursor.close()

            transaction.commit_unless_managed()

            run.committed = True
            run.save()

        for house in houses_to_consolidate:
            call_command("consolidate", house)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Estêvão Samuel Procópio
# Copyright (©) 2010-2013 Gustavo Noronha Silva
# Copyright (©) 2014 Lúcio Flávio Corrêa
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import requests
import subprocess
from datetime import date, datetime
from django.db import reset_queries
from email.utils import formatdate as http_date
from basecollector import BaseCollector
from lxml.etree import iterparse
from montanha.models import (ArchivedExpense, Institution, Legislature,
                             Legislator, AlternativeLegislatorName, Mandate,
                             ExpenseNature, Supplier, PoliticalParty, CollectionRun)
from zipfile import ZipFile


OBJECT_LIST_MAXIMUM_COUNTER = 1000


class CamaraDosDeputados(BaseCollector):
    def __init__(self, collection_runs, debug_enabled=False):
        super(CamaraDosDeputados, self).__init__(collection_runs, debug_enabled)

        institution, _ = Institution.objects.get_or_create(siglum='CDF', name=u'Câmara dos Deputados Federais')
        self.legislature, _ = Legislature.objects.get_or_create(institution=institution,
                                                                date_start=datetime(2015, 1, 1),
                                                                date_end=datetime(2018, 12, 31))

    def retrieve_legislators(self):
        uri = 'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/ObterDeputados'
        return BaseCollector.retrieve_uri(self, uri)

    def try_name_disambiguation(self, name):
        if name.title() == 'Sergio Souza':
            return Legislator.objects.get(id=293), False

        return None, False

    def update_legislators(self):
        xml = self.retrieve_legislators()

        for l in xml.findAll('deputado'):
            alternative_name = None
            name = l.find('nomeparlamentar')
            if not name:
                name = l.find('nome')
            else:
                alternative_name = l.find('nome').text.title().strip()

            name = name.text.title().strip()

            self.debug(u"Looking for legislator: %s" % unicode(name))
            legislator, created = Legislator.objects.get_or_create(name=name)

            if created:
                self.debug(u"New legislator: %s" % unicode(legislator))
            else:
                self.debug(u"Found existing legislator: %s" % unicode(legislator))

            if alternative_name:
                try:
                    legislator.alternative_names.get(name=alternative_name)
                except AlternativeLegislatorName.DoesNotExist:
                    alternative_name, _ = AlternativeLegislatorName.objects.get_or_create(name=alternative_name)
                    legislator.alternative_name = alternative_name

            legislator.email = l.find('email').text
            legislator.save()

            party_name = self._normalize_party_name(l.find('partido').text)
            party, _ = PoliticalParty.objects.get_or_create(siglum=party_name)

            state = l.find('uf').text

            original_id = l.find('ideCadastro')

            mandate = self.mandate_for_legislator(legislator, party,
                                                  state=state, original_id=original_id)

    def update_data(self):
        if os.path.exists('cdep-collection-run'):
            crid = int(open('cdep-collection-run').read())
            CollectionRun.objects.get(id=crid).delete()
            os.unlink('cdep-collection-run')

        self.collection_run = self.create_collection_run(self.legislature)

        data_path = os.path.join(os.getcwd(), 'data', 'cdep')

        files_to_download = ['AnoAtual.zip']
        previous_years = date.today().year - self.legislature.date_start.year

        if previous_years:
            files_to_download.append('AnoAnterior.zip')

        if previous_years > 1:
            files_to_download.append('AnosAnteriores.zip')

        files_to_process = list()
        for file_name in files_to_download:
            xml_file_name = file_name.replace('zip', 'xml')
            full_xml_path = os.path.join(data_path, xml_file_name)
            files_to_process.append(os.path.join(data_path, full_xml_path))

            full_path = os.path.join(data_path, file_name)

            headers = dict()
            if os.path.exists(full_path):
                headers['If-Modified-Since'] = http_date(os.path.getmtime(full_path), usegmt=True)

            uri = 'http://www.camara.gov.br/cotas/' + file_name
            self.debug(u"Preparing to download %s…" % (uri))
            r = requests.get(uri, headers=headers, stream=True)

            if r.status_code == requests.codes.not_modified:
                self.debug(u"File %s not updated since last download, skipping…" % file_name)
                continue

            with open(full_path, 'wb') as f:
                for chunk in r.iter_content(chunk_size=8192):
                    if chunk:
                        f.write(chunk)

            self.debug(u"Unzipping %s…" % (file_name))
            zf = ZipFile(full_path, 'r')
            zf.extract(xml_file_name, data_path)

        open('cdep-collection-run', 'w').write('%d' % (self.collection_run.id))
        archived_expense_list = []
        for file_name in reversed(files_to_process):
            self.debug(u"Processing %s…" % file_name)
            objects_counter = 0

            context = iterparse(file_name, events=("start", "end"))

            # turn it into an iterator
            context = iter(context)

            for event, elem in context:
                if event != "end" or elem.tag != "DESPESA":
                    continue

                # Some entries lack numLegislatura, so we fallback to numAno.
                legislature_year = elem.find('nuLegislatura')
                if legislature_year is not None:
                    legislature_year = int(legislature_year.text)
                else:
                    legislature_year = int(elem.find('numAno').text)
                    if legislature_year < self.legislature.date_start.year or \
                       legislature_year > self.legislature.date_end.year:
                        legislature_year = None
                    else:
                        legislature_year = self.legislature.date_start.year

                if legislature_year != self.legislature.date_start.year:
                    self.debug(u"Ignoring entry because it's out of the target legislature…")
                    continue

                name = elem.find('txNomeParlamentar').text.title().strip()

                nature = elem.find('txtDescricao').text.title().strip()

                supplier_name = elem.find('txtBeneficiario')
                if supplier_name is not None:
                    supplier_name = supplier_name.text.title().strip()
                else:
                    supplier_name = u'Sem nome'

                supplier_identifier = elem.find('txtCNPJCPF')
                if supplier_identifier is not None and supplier_identifier.text is not None:
                    supplier_identifier = self.normalize_cnpj_or_cpf(supplier_identifier.text)

                if not supplier_identifier:
                    supplier_identifier = u'Sem CNPJ/CPF (%s)' % supplier_name

                try:
                    supplier = Supplier.objects.get(identifier=supplier_identifier)
                except Supplier.DoesNotExist:
                    supplier = Supplier(identifier=supplier_identifier, name=supplier_name)
                    supplier.save()

                docnumber = elem.find('txtNumero').text
                if docnumber:
                    docnumber = docnumber.strip()

                expense_date = elem.find('datEmissao')
                if expense_date is not None:
                    expense_date = date(*((int(x.lstrip('0')) for x in expense_date.text[:10].split('-'))))
                else:
                    expense_year = int(elem.find('numAno').text)
                    expense_month = int(elem.find('numMes').text)
                    expense_date = date(expense_year, expense_month, 1)

                expensed = float(elem.find('vlrLiquido').text)

                nature, _ = ExpenseNature.objects.get_or_create(name=nature)

                party = party_name = elem.find('sgPartido')
                if party_name is not None:
                    party_name = self._normalize_party_name(party_name.text)
                    party, _ = PoliticalParty.objects.get_or_create(siglum=party_name)

                state = elem.find('sgUF').text.strip()

                original_id = elem.find('ideCadastro').text.strip()

                try:
                    legislator = Legislator.objects.get(name__iexact=name)
                except Legislator.DoesNotExist:
                    # Some legislators do are not listed in the other WS because they are not
                    # in exercise.
                    self.debug(u"Found legislator who's not in exercise: %s" % name)
                    legislator = Legislator(name=name)
                    legislator.save()
                mandate = self.mandate_for_legislator(legislator, party,
                                                      state=state, original_id=original_id)
                expense = ArchivedExpense(number=docnumber,
                                          nature=nature,
                                          date=expense_date,
                                          expensed=expensed,
                                          mandate=mandate,
                                          supplier=supplier,
                                          collection_run=self.collection_run)
                archived_expense_list.append(expense)
                self.debug(u"New expense found: %s" % unicode(expense))

                objects_counter += 1

                if objects_counter == OBJECT_LIST_MAXIMUM_COUNTER:
                    ArchivedExpense.objects.bulk_create(archived_expense_list)
                    archived_expense_list[:] = []
                    objects_counter = 0
                    reset_queries()

                elem.clear()
                while elem.getprevious() is not None:
                    del elem.getparent()[0]

                continue

        if archived_expense_list:
            ArchivedExpense.objects.bulk_create(archived_expense_list)

        os.unlink('cdep-collection-run')

    def _normalize_name(self, name):
        names_map = {
            'Gim': 'Gim Argello',
        }
        return names_map.get(name, name)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2014, Marcelo Jorge Vieira <metal@alucinados.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from StringIO import StringIO

from django.core.management import call_command
from django.test import TestCase


class CollectCommandsTestCase(TestCase):

    def test_command_collect_without_instituiton(self):
        out = StringIO()
        call_command('collect', stdout=out)
        self.assertEqual(out.getvalue(), '')



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'CollectionRun.committed'
        db.add_column(u'montanha_collectionrun', 'committed',
                      self.gf('django.db.models.fields.BooleanField')(default=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'CollectionRun.committed'
        db.delete_column(u'montanha_collectionrun', 'committed')


    models = {
        u'montanha.alternativelegislatorname': {
            'Meta': {'object_name': 'AlternativeLegislatorName'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '2048'})
        },
        u'montanha.archivedexpense': {
            'Meta': {'object_name': 'ArchivedExpense'},
            'collection_run': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.CollectionRun']"}),
            'date': ('django.db.models.fields.DateField', [], {}),
            'expensed': ('django.db.models.fields.DecimalField', [], {'max_digits': '10', 'decimal_places': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mandate': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Mandate']"}),
            'nature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.ExpenseNature']"}),
            'number': ('django.db.models.fields.CharField', [], {'max_length': '512'}),
            'original_id': ('django.db.models.fields.CharField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'}),
            'supplier': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Supplier']"}),
            'value': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '10', 'decimal_places': '2', 'blank': 'True'})
        },
        u'montanha.biggestsupplierforyear': {
            'Meta': {'object_name': 'BiggestSupplierForYear'},
            'expensed': ('django.db.models.fields.DecimalField', [], {'max_digits': '20', 'decimal_places': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'supplier': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Supplier']"}),
            'year': ('django.db.models.fields.IntegerField', [], {})
        },
        u'montanha.collectionrun': {
            'Meta': {'object_name': 'CollectionRun'},
            'committed': ('django.db.models.fields.BooleanField', [], {}),
            'date': ('django.db.models.fields.DateField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'legislature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Legislature']"})
        },
        u'montanha.expense': {
            'Meta': {'object_name': 'Expense'},
            'date': ('django.db.models.fields.DateField', [], {}),
            'expensed': ('django.db.models.fields.DecimalField', [], {'max_digits': '10', 'decimal_places': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mandate': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Mandate']"}),
            'nature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.ExpenseNature']"}),
            'number': ('django.db.models.fields.CharField', [], {'max_length': '512'}),
            'original_id': ('django.db.models.fields.CharField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'}),
            'supplier': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Supplier']"}),
            'value': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '10', 'decimal_places': '2', 'blank': 'True'})
        },
        u'montanha.expensenature': {
            'Meta': {'object_name': 'ExpenseNature'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '512'}),
            'original_id': ('django.db.models.fields.CharField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'})
        },
        u'montanha.institution': {
            'Meta': {'object_name': 'Institution'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'logo': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '2048'}),
            'siglum': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '10'})
        },
        u'montanha.legislator': {
            'Meta': {'object_name': 'Legislator'},
            'about': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'alternative_names': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['montanha.AlternativeLegislatorName']", 'symmetrical': 'False'}),
            'date_of_birth': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '1', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '2048'}),
            'picture': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        u'montanha.legislature': {
            'Meta': {'object_name': 'Legislature'},
            'date_end': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_start': ('django.db.models.fields.DateField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Institution']"}),
            'original_id': ('django.db.models.fields.CharField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'})
        },
        u'montanha.mandate': {
            'Meta': {'object_name': 'Mandate'},
            'date_end': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_start': ('django.db.models.fields.DateField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'legislator': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Legislator']"}),
            'legislature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Legislature']"}),
            'original_id': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'party': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.PoliticalParty']", 'null': 'True', 'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'})
        },
        u'montanha.perlegislator': {
            'Meta': {'object_name': 'PerLegislator'},
            'date_end': ('django.db.models.fields.DateField', [], {}),
            'date_start': ('django.db.models.fields.DateField', [], {}),
            'expensed': ('django.db.models.fields.DecimalField', [], {'max_digits': '20', 'decimal_places': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Institution']"}),
            'legislator': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Legislator']"}),
            'legislature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Legislature']", 'null': 'True', 'blank': 'True'})
        },
        u'montanha.pernature': {
            'Meta': {'object_name': 'PerNature'},
            'date_end': ('django.db.models.fields.DateField', [], {}),
            'date_start': ('django.db.models.fields.DateField', [], {}),
            'expensed': ('django.db.models.fields.DecimalField', [], {'max_digits': '20', 'decimal_places': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Institution']"}),
            'legislature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Legislature']", 'null': 'True', 'blank': 'True'}),
            'nature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.ExpenseNature']"})
        },
        u'montanha.pernaturebymonth': {
            'Meta': {'object_name': 'PerNatureByMonth'},
            'date': ('django.db.models.fields.DateField', [], {}),
            'expensed': ('django.db.models.fields.DecimalField', [], {'max_digits': '20', 'decimal_places': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Institution']"}),
            'nature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.ExpenseNature']"})
        },
        u'montanha.pernaturebyyear': {
            'Meta': {'object_name': 'PerNatureByYear'},
            'expensed': ('django.db.models.fields.DecimalField', [], {'max_digits': '20', 'decimal_places': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.Institution']"}),
            'nature': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['montanha.ExpenseNature']"}),
            'year': ('django.db.models.fields.IntegerField', [], {})
        },
        u'montanha.politicalparty': {
            'Meta': {'object_name': 'PoliticalParty'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'logo': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '2048', 'null': 'True', 'blank': 'True'}),
            'siglum': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '10'}),
            'wikipedia': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        u'montanha.supplier': {
            'Meta': {'object_name': 'Supplier'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        }
    }

    complete_apps = ['montanha']


# -*- coding: utf-8 -*-
#
# Copyright (©) 2013 Marcelo Jorge Vieira <metal@alucinados.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import urllib
from datetime import datetime
from django.core.files import File

from basecollector import BaseCollector
from montanha.models import *


def parse_money(string):
    string = string.replace('.', '')
    string = string.replace(',', '.')
    return float(string)


def parse_date(string):
    return datetime.strptime(string, '%d/%m/%Y').date()


def parse_cmsp_date(month, year):
    day = '01'
    return parse_date(day + '/' + month + '/' + year)


class CMSP(BaseCollector):
    def __init__(self, collection_runs, debug_enabled=False):
        super(CMSP, self).__init__(collection_runs, debug_enabled)

        institution, institution_created = Institution.objects.get_or_create(
            siglum='CMSP',
            name=u'Câmara Municipal de São Paulo')

        self.institution = institution

    def _normalize_party_siglum(self, siglum):
        names_map = {
            'DEMOCRATAS': 'DEM',
        }
        return names_map.get(siglum, siglum)

    def retrieve_expenses(self, month, year):
        uri = 'http://www2.camara.sp.gov.br/SAEG/%s%s.xml' % (year, month)
        return BaseCollector.retrieve_uri(self, uri, force_encoding='utf-8')

    def retrieve_legislators(self):
        uri = 'http://www1.camara.sp.gov.br/vereadores_joomla.asp'
        return BaseCollector.retrieve_uri(self, uri)

    def retrieve_legislator(self, link):
        uri = 'http://www1.camara.sp.gov.br/%s' % link
        return BaseCollector.retrieve_uri(self, uri)

    def add_legislator(self, name):
        legislator, created = Legislator.objects.get_or_create(name=name)

        if created:
            self.debug(u'New legislator found: %s' % legislator)
        else:
            self.debug(u'Found existing legislator: %s' % legislator)

        return legislator

    def process_legislators(self, legislature):
        legislators = self.retrieve_legislators()
        if not legislators:
            return

        links = legislators.findAll(
            'a',
            href=re.compile('^vereador_joomla2.asp\?vereador='))

        for link in links:
            href = link.get('href')
            html_legislator = self.retrieve_legislator(href)
            if not html_legislator:
                continue

            url, code = href.split('=', 1)
            name = html_legislator.find(id='nome_vereador').getText()

            legislator = self.add_legislator(name)

            legislator_img = html_legislator.find(
                'img',
                src=re.compile('imgs/fotos/'))

            if legislator_img:
                legislator_img_src = legislator_img.get('src')

                legislator_img_url = 'http://www1.camara.sp.gov.br/%s' % (
                    legislator_img_src)

                result = urllib.urlretrieve(legislator_img_url)

                legislator.picture.save(
                    os.path.basename(legislator_img_url), File(open(result[0])))

                legislator.save()

                self.debug('Updating legislator picture.')

            try:
                mandate = Mandate.objects.get(
                    legislator=legislator,
                    date_start=legislature.date_start,
                    legislature=legislature)
                self.debug(u'Found existing Mandate: %s' % mandate)
            except Mandate.DoesNotExist:
                mandate = Mandate(
                    legislator=legislator,
                    date_start=legislature.date_start,
                    legislature=legislature)
                mandate.save()
                self.debug(u'New Mandate found: %s' % mandate)

            party_name = html_legislator.find(
                'img',
                src=re.compile('imgs/Partidos'))

            party_name = party_name.parent.parent.find('font', size='2')
            party_name = party_name.getText()
            party_siglum = party_name[party_name.find('(')+1:party_name.find(')')]

            if 'Vereadores Licenciados' not in party_siglum:
                party_siglum = self._normalize_party_siglum(party_siglum)
                party, party_created = PoliticalParty.objects.get_or_create(
                    siglum=party_siglum)

                mandate.party = party
                mandate.save()
                self.debug('Updating legislator party: %s' % party_siglum)

    def process_expenses(self, month, year, legislature, collection_run):
        data = self.retrieve_expenses(month, year)
        if not data:
            return

        for x in data.findAll('g_deputado'):
            name = x.find('nm_deputado').getText().capitalize()
            legislator = self.add_legislator(name)

            try:
                mandate = Mandate.objects.get(
                    legislator=legislator,
                    date_start=legislature.date_start,
                    legislature=legislature)
                self.debug(u'Found existing Mandate: %s' % mandate)
            except Mandate.DoesNotExist:
                mandate = Mandate(
                    legislator=legislator,
                    date_start=legislature.date_start,
                    legislature=legislature)
                mandate.save()
                self.debug(u'New Mandate found: %s' % mandate)

            expense_type = x.find('list_g_tipo_despesa')

            for i in expense_type.findAll('g_tipo_despesa'):
                nature_text = i.find('nm_tipo_despesa').getText()
                try:
                    nature_text = nature_text.split('-', 1)[1].strip()
                except IndexError:
                    pass

                nature_text = nature_text.capitalize()

                ignore_list = [u'total', u'TOTAL', u'utilizado até 30/11/07']
                ignore_matches = [s for s in ignore_list if s in nature_text]
                if ignore_matches:
                    continue

                nature, nature_created = ExpenseNature.objects.get_or_create(
                    name=nature_text)

                if nature_created:
                    self.debug(u'New ExpenseNature found: %s' % nature)
                else:
                    self.debug(u'Found existing ExpenseNature: %s' % nature)

                m_month = i.find('nr_mes_ref').getText()
                m_year = i.find('nr_ano_ref').getText()
                date = parse_cmsp_date(m_month, m_year)

                for j in i.findAll('g_beneficiario'):
                    supplier_name = j.find('nm_beneficiario').getText()
                    supplier_name = supplier_name.capitalize()
                    cnpj = self.normalize_cnpj_or_cpf(j.find('nr_cnpj').getText())

                    if not cnpj and not supplier_name:
                        continue

                    try:
                        supplier = Supplier.objects.get(identifier=cnpj)
                        supplier_created = False
                    except Supplier.DoesNotExist:
                        supplier = Supplier(identifier=cnpj, name=supplier_name)
                        supplier.save()
                        supplier_created = True

                    if supplier_created:
                        self.debug(u'New Supplier found: %s' % supplier)
                    else:
                        self.debug(u'Found existing supplier: %s' % supplier)

                    expensed = parse_money(j.find('vl_desp').getText())

                    expense = ArchivedExpense(number='None',
                                              nature=nature,
                                              date=date,
                                              expensed=expensed,
                                              mandate=mandate,
                                              supplier=supplier,
                                              collection_run=collection_run)
                    expense.save()

                    self.debug(u'New expense found: %s' % expense)

    def get_legislature(self, year):
        start_year = end_year = year

        if year >= 2013 and year <= 2016:
            start_year = 2013
            end_year = 2016

        if year >= 2009 and year <= 2012:
            start_year = 2009
            end_year = 2012

        if year >= 2005 and year <= 2008:
            start_year = 2005
            end_year = 2008

        legislature, created = Legislature.objects.get_or_create(
            institution=self.institution,
            date_start=datetime(start_year, 1, 1),
            date_end=datetime(end_year, 12, 31))

        if created:
            self.debug(u'New Legislature found: %s' % legislature)
        else:
            self.debug(u'Found existing legislature: %s' % legislature)

        return legislature

    def process_all_expenses(self):
        for year in xrange(2007, 2014):
            legislature = self.get_legislature(year)
            collection_run = self.create_collection_run(legislature)
            for m in xrange(1, 13):
                month = '%02d' % m
                self.debug('Adding expenses from %s/%s' % (month, year))
                self.process_expenses(month, year, legislature, collection_run)

    def process_current_legislators(self):
        current_legislature = self.get_legislature(2013)
        self.process_legislators(current_legislature)

    def process_all_legislators(self):
        import urllib2
        uri = 'http://www2.camara.sp.gov.br/Dados_abertos/vereador/vereador.txt'

        contents = urllib2.urlopen(uri)

        for line in contents:

            try:
                line = line.decode('utf-8')
            except UnicodeDecodeError:
                try:
                    line = line.decode('iso-8859-1')
                except Exception:
                    pass

            row = line.split('#')
            if len(row) > 1:
                name = row[1]
                nickname = row[2]

                if nickname:
                    legislator = self.add_legislator(nickname)
                else:
                    legislator = self.add_legislator(name)

                aa = re.search('\^\p([^\^|%]*)(\^|%)', row[7])
                if aa:
                    party_siglum = aa.group(1)
                else:
                    pass

                try:
                    date_start_re = re.search('\^\i([^\^|%]*)(\^|%)', row[7])
                    start_year = int(date_start_re.group(1).split('/')[2])
                    date_start = datetime(start_year, 1, 1)
                except IndexError:
                    continue

                legislature = self.get_legislature(start_year)

                mandate, mandate_created = Mandate.objects.get_or_create(
                    legislator=legislator,
                    date_start=legislature.date_start,
                    legislature=legislature)

                if mandate_created:
                    self.debug(u'New Mandate found: %s' % mandate)
                else:
                    self.debug(u'Found existing Mandate: %s' % mandate)

                if party_siglum and 'Sem' or 'Vereaores' not in party_siglum:
                    party, party_created = PoliticalParty.objects.get_or_create(
                        siglum=party_siglum)

                    mandate.party = party
                    mandate.save()
                    self.debug('Updating legislator party: %s' % party)

    def update_data(self):
        self.process_all_legislators()
        self.process_current_legislators()
        self.process_all_expenses()



# Downloaded from http://wiki.python.org.br/Cpf
# License: CC-BY http://creativecommons.org/licenses/by/2.5/br/


class Cpf:
    def __init__( self ):
        """
        Class to interact with CPF numbers
        """
        pass

    def format( self, cpf ):
        """
        Method that formats a brazilian CPF

        Tests:
        >>> print Cpf().format('91289037736')
        912.890.377-36
        """
        return "%s.%s.%s-%s" % ( cpf[0:3], cpf[3:6], cpf[6:9], cpf[9:11] )

    def validate( self, cpf ):
        """
        Method to validate a brazilian CPF number
        Based on Pedro Werneck source avaiable at
        www.PythonBrasil.com.br

        Tests:
        >>> print Cpf().validate('91289037736')
        True
        >>> print Cpf().validate('91289037731')
        False
        """
        cpf_invalidos = [11*str(i) for i in range(10)]
        if cpf in cpf_invalidos:
            return False

        if not cpf.isdigit():
            """ Verifica se o CPF contem pontos e hifens """
            cpf = cpf.replace( ".", "" )
            cpf = cpf.replace( "-", "" )

        if len( cpf ) < 11:
            """ Verifica se o CPF tem 11 digitos """
            return False

        if len( cpf ) > 11:
            """ CPF tem que ter 11 digitos """
            return False

        selfcpf = [int( x ) for x in cpf]

        cpf = selfcpf[:9]

        while len( cpf ) < 11:

            r =  sum( [( len( cpf )+1-i )*v for i, v in [( x, cpf[x] ) for x in range( len( cpf ) )]] ) % 11

            if r > 1:
                f = 11 - r
            else:
                f = 0
            cpf.append( f )


        return bool( cpf == selfcpf )




# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Estêvão Samuel Procópio
# Copyright (©) 2010-2013 Gustavo Noronha Silva
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import base64
from basecollector import BaseCollector
from datetime import datetime, date
from montanha.models import *


def parse_money(string):
    string = string.strip('R$ ')
    string = string.replace('.', '')
    string = string.replace(',', '.')
    return float(string)


def parse_date(string):
    return datetime.strptime(string, '%d/%m/%Y').date()


def parse_cmbh_date(date_string):
    day = '01'
    month, year = date_string.split('/')
    return parse_date(day + '/' + month + '/' + year)


class CMBH(BaseCollector):
    def __init__(self, collection_runs, debug_enabled=False):
        super(CMBH, self).__init__(collection_runs, debug_enabled)

        try:
            institution = Institution.objects.get(siglum='CMBH')
        except Institution.DoesNotExist:
            institution = Institution(siglum='CMBH', name=u'Câmara Municipal de Belo Horizonte')
            institution.save()

        try:
            self.legislature = Legislature.objects.all().filter(institution=institution).order_by('-date_start')[0]
        except IndexError:
            self.legislature = Legislature(institution=institution,
                                           date_start=datetime(2013, 1, 1),
                                           date_end=datetime(2016, 12, 31))
            self.legislature.save()

    def retrieve_months(self):
        uri = 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/lista_meses.php'
        data = {'tipo': 'd'}
        headers = {
            'Referer': 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/index.php',
            'Origin': 'http://www.cmbh.mg.gov.br',
        }
        return BaseCollector.retrieve_uri(self, uri, data, headers)

    def retrieve_legislators(self, month):
        uri = 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/oracle_lista_vereadores.php'
        data = {'mes': month}
        headers = {
            'Referer': 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/index.php',
            'Origin': 'http://www.cmbh.mg.gov.br',
        }
        return BaseCollector.retrieve_uri(self, uri, data, headers)

    def retrieve_expense_types(self, month, legislator, code):
        uri = 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/oracle_lista_tipodespesa.php'
        data = {'mes': month, 'vereador': legislator, 'cod': code}
        headers = {
            'Referer': 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/oracle_lista_vereadores.php',
            'Origin': 'http://www.cmbh.mg.gov.br',
        }
        return BaseCollector.retrieve_uri(self, uri, data, headers)

    def retrieve_actual_data(self, code, seq, legislator, nature, month):
        uri = 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/oracle_lista_valordespesa.php'
        data = {
            'cod': code,
            'seq': seq,
            'vereador': legislator,
            'tipodespesa': nature,
            'mes': month
        }
        headers = {
            'Referer': 'http://www.cmbh.mg.gov.br/extras/verba_indenizatoria_nota_fiscal/oracle_lista_tipodespesa.php',
            'Origin': 'http://www.cmbh.mg.gov.br',
        }
        return BaseCollector.retrieve_uri(self, uri, data, headers)

    def update_legislators(self):
        pass

    def _normalize_nature(self, nature):
        if not hasattr(self, 'nature_map'):
            self.nature_map = {
                'Servico ou Produto Postal': 'Serviço ou Produto Postal',
                'Periodico': 'Periódico',
                'Material de Escritorio': 'Material de Escritório',
                'Material de Informatica': 'Material de Informática',
                'Servico de Escritorio': 'Serviço de Escritório',
                'Servico de Informatica': 'Serviço de Informática',
                'Estacionamento': 'Estacionamento',
                'Lanche': 'Lanche',
                'Refeic?o': 'Refeicão',
                'Telecomunicac?o': 'Telecomunicação',
                'Combustivel': 'Combustivel',
                'Manutenc?o e Locac?o de Veiculo': 'Manutenção e Locação de Veiculo',
                'Participac?o em Curso ou Seminario': 'Participação em Curso ou Seminario',
                'Viagem a Servico': 'Viagem a Serviço',
                'Consultoria Tecnico-Especializada': 'Consultoria Técnico-Especializada',
                'Apoio a Promoc?o de Eventos Oficiais': 'Apoio a Promoção de Eventos Oficiais',
                'Escritorio Representac?o Parlamentar': 'Escritorio Representação Parlamentar',
                'Servico Grafico': 'Serviço Gráfico',
                'Divulgac?o de Atividade Parlamentar': 'Divulgação de Atividade Parlamentar'
            }

        return self.nature_map.get(nature, nature)

    def update_data_for_legislator(self, month, legislator, code):
        expense_types = self.retrieve_expense_types(month, legislator, code)

        if not expense_types:
            return

        # Ignore the last one, which is the total.
        expense_types = expense_types.find('ul').findAll('a')[:-1]

        parameters_list = []
        for etype in expense_types:
            parts = etype['onclick'].split("'")
            legislator = parts[1]
            code = parts[3]
            nature = parts[5]
            seq = parts[7]
            month = parts[9]

            data = self.retrieve_actual_data(code, seq, legislator, nature, month)

            if not data:
                print 'No data...'
                continue

            # Get the lines of data, ignoring the first one, which
            # contains the titles, and the last one, which contains
            # the total.
            data = data.find('div', 'texto_valores1').findAll('tr')[1:-1]

            if not data:
                continue

            legislator = base64.decodestring(legislator).strip().decode('utf-8')
            date = parse_cmbh_date(base64.decodestring(month).strip().decode('utf-8'))

            nature = base64.decodestring(nature).strip().decode('utf-8')
            nature = self._normalize_nature(nature)
            try:
                nature = ExpenseNature.objects.get(name=nature)
            except ExpenseNature.DoesNotExist:
                nature = ExpenseNature(name=nature)
                nature.save()

            legislator, created = Legislator.objects.get_or_create(name=legislator)

            if created:
                self.debug("New legislator: %s" % unicode(legislator))
            else:
                self.debug("Found existing legislator: %s" % unicode(legislator))

            mandate = self.mandate_for_legislator(legislator, party=None, original_id=code)

            for row in data:
                columns = row.findAll('td')

                if not len(columns) == 5:
                    print u'Bad row: %s' % unicode(columns)
                    continue

                cnpj = self.normalize_cnpj_or_cpf(columns[0].getText())

                supplier_name = columns[1].getText().strip()

                try:
                    supplier_name = supplier_name.decode('utf-8')
                except Exception:
                    pass

                try:
                    supplier = Supplier.objects.get(identifier=cnpj)
                except Supplier.DoesNotExist:
                    supplier = Supplier(identifier=cnpj, name=supplier_name)
                    supplier.save()

                docnumber = columns[2].getText()
                expensed = parse_money(columns[3].getText())

                expense = ArchivedExpense(number=docnumber,
                                          nature=nature,
                                          date=date,
                                          expensed=expensed,
                                          mandate=mandate,
                                          supplier=supplier,
                                          collection_run=self.collection_run)
                expense.save()

                self.debug("New expense found: %s" % unicode(expense))

    def update_data(self):
        self.collection_run = self.create_collection_run(self.legislature)
        for year in range(self.legislature.date_start.year, datetime.now().year + 1):
            self.update_data_for_year(year)

    def update_data_for_year(self, year=datetime.now().year):
        self.debug("Updating data for year %d" % year)
        months = self.retrieve_months().findAll('div', 'arquivo_meses')

        date_list = []
        for month in months:
            anchor = month.find('a')
            parts = anchor['onclick'].split("'")
            date_list.append(parts[1])

        for date in date_list:
            leg_list = self.retrieve_legislators(date)
            anchors = leg_list.find('ul').findAll('a')
            for anchor in anchors:
                parts = anchor['onclick'].split("'")
                legislator = parts[1]
                code = parts[3]
                month = parts[5]
                self.update_data_for_legislator(month, legislator, code)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2013 Marcelo Jorge Vieira <metal@alucinados.com>
# Copyright (©) 2013 Gustavo Noronha Silva <gustavo@noronha.eti.br>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import date
from montanha.models import Institution, Legislature


def filter_for_institution(data, institution):
    if not institution:
        return data

    if not isinstance(institution, Institution):
        institution = Institution.objects.get(siglum=institution)

    data = data.filter(mandate__legislature__institution=institution)
    return data


def get_date_ranges_from_data(institution, data, consolidated_data=False, include_date_objects=True):
    """ Takes a data set and returns a dict containing in textual form:

        current_date_from: the start date that is being used for this query
        current_date_to: the end date that is being used for this query
    """
    try:
        if consolidated_data:
            cdf = data.order_by('date_start')[0].date_start
        else:
            cdf = data.order_by('date')[0].date
    except:
        cdf = date.today()

    try:
        if consolidated_data:
            cdt = data.order_by('-date_end')[0].date_end
        else:
            cdt = data.order_by('-date')[0].date
    except:
        cdt = date.today()

    if institution:
        if not isinstance(institution, Institution):
            institution = Institution.objects.get(siglum=institution)

        # Bound dates to the start of the first legislature to the end
        # of the last, which makes more sense to our purposes.
        first = institution.legislature_set.order_by('date_start')[0]
        last = institution.legislature_set.order_by('-date_end')[0]
    else:
        first = Legislature.objects.order_by('date_start')[0]
        last = Legislature.objects.order_by('-date_end')[0]

    min_date = first.date_start
    max_date = last.date_end

    if cdf < min_date:
        cdf = min_date

    if cdt > max_date:
        cdt = max_date

    cdf_string = cdf.strftime('%B de %Y')
    cdt_string = cdt.strftime('%B de %Y')

    d = dict(current_date_from=cdf_string, current_date_to=cdt_string)
    if include_date_objects:
        d.update(dict(cdf=cdf, cdt=cdt))

    return d


def ensure_years_in_range(date_ranges, years):
    nyears = []
    cdf = date_ranges['cdf']
    cdt = date_ranges['cdt']
    for y in years:
        d = date(y, 1, 1)
        if d < cdf or d > cdt:
            continue
        nyears.append(y)
    return nyears



# -*- coding: utf-8 -*-
#
# Copyright (©) 2014, Marcelo Jorge Vieira <metal@alucinados.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from django.test.client import RequestFactory

from montanha.views import show_index, show_robots_txt
from montanha.tests.fixtures import InstitutionFactory


class MontanhaViewTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_render_index_without_institution(self):
        try:
            request = self.factory.get('/')
            response = show_index(request)
        except TypeError as e:
            response = None
            msg = 'show_index() takes exactly 2 arguments (1 given)'
            self.assertEqual(e.message, msg)

        self.assertEqual(response, None)

    def test_render_index_with_empty_institution(self):
        request = self.factory.get('/')
        response = show_index(request, '')
        self.assertEqual(response.status_code, 200)
        self.assertTrue('O olho neles é' in response.content)

    def test_render_index(self):
        institution = InstitutionFactory.create()

        request = self.factory.get('/')
        response = show_index(request, institution.siglum)
        self.assertEqual(response.status_code, 200)
        self.assertFalse('O olho neles é' in response.content)
        self.assertIn(institution.siglum, response.content)

    def test_render_robots_txt(self):
        request = self.factory.get('/robots.txt')
        response = show_robots_txt(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, 'User-Agent: *\nAllow: /\n')



# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Gustavo Noronha Silva
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext as _
from easy_thumbnails.fields import ThumbnailerImageField

from montanha.cpf import Cpf
from montanha.cnpj import Cnpj


class PoliticalParty(models.Model):

    class Meta:
        verbose_name = _("Political Party")
        verbose_name_plural = _("Political Parties")

    siglum = models.CharField(max_length=15,
                              verbose_name=_("Siglum"),
                              unique=True)

    name = models.CharField(max_length=2048,
                            verbose_name=_("Full name"),
                            blank=True,
                            null=True)

    logo = ThumbnailerImageField(
        verbose_name=_('Logo'),
        upload_to='political_party',
        blank=True,
        null=True)

    wikipedia = models.URLField(
        verbose_name=_('Wikipedia'),
        blank=True,
        null=True)

    def __unicode__(self):
        return u"%s" % (self.siglum)


class AlternativeLegislatorName(models.Model):
    name = models.CharField(max_length=2048)

    def __unicode__(self):
        return u"%s" % (self.name)


class Legislator(models.Model):

    class Meta:
        verbose_name = _("Legislator")
        verbose_name_plural = _("Legislators")

    name = models.CharField(max_length=2048,
                            verbose_name=_("Full name"))

    alternative_names = models.ManyToManyField('AlternativeLegislatorName')

    picture = ThumbnailerImageField(
        verbose_name=_('Picture'),
        upload_to='legislator',
        blank=True,
        null=True)

    site = models.URLField(
        verbose_name=_('Site'),
        blank=True,
        null=True)

    email = models.EmailField(
        verbose_name=_('Email'),
        blank=True,
        null=True)

    about = models.TextField(
        verbose_name=_('About'),
        blank=True,
        null=True)

    gender = models.CharField(
        verbose_name=_('Gender'),
        choices=(
            ('F', _('Female')),
            ('M', _('Male'))),
        blank=True,
        null=True,
        max_length=1)

    date_of_birth = models.DateField(
        verbose_name=_('Date of Birth'),
        blank=True,
        null=True)

    def __unicode__(self):
        return u"Legislator %s" % (self.name)

    @property
    def party(self):
        return self.mandate_set.order_by("-date_start")[0].party


class Institution(models.Model):

    class Meta:
        verbose_name = _("Institution")
        verbose_name_plural = _("Institutions")

    name = models.CharField(max_length=2048,
                            verbose_name=_("Name"))

    siglum = models.CharField(max_length=10,
                              verbose_name=_("Siglum"),
                              unique=True)

    logo = ThumbnailerImageField(
        verbose_name=_('Logo'),
        upload_to='institution',
        blank=True,
        null=True)

    def __unicode__(self):
        return u"%s" % (self.name)


class Legislature(models.Model):

    class Meta:
        verbose_name = _("Legislature")
        verbose_name_plural = _("Legislatures")

    institution = models.ForeignKey("Institution")

    original_id = models.CharField(blank=True, null=True,
                                   max_length=512,
                                   verbose_name=_("Original ID"))

    date_start = models.DateField(verbose_name=_("Date started"),
                                  help_text=_("""Date in which this legislature started."""))

    date_end = models.DateField(blank=True, null=True,
                                verbose_name=_("Date ended"),
                                help_text=_("""Date in which this legislature ended."""))

    def __unicode__(self):
        return u"%s's legislature starting at %s, ending at %s" % (self.institution.siglum,
                                                                   str(self.date_start),
                                                                   str(self.date_end))


class Mandate(models.Model):

    class Meta:
        verbose_name = _("Mandate")
        verbose_name_plural = _("Mandates")

    original_id = models.TextField(null=True, blank=True)

    legislator = models.ForeignKey("Legislator")

    legislature = models.ForeignKey("Legislature")

    date_start = models.DateField(verbose_name=_("Date started"),
                                  help_text=_("""Date in which this mandate started; may also be """
                                              """a resumption of a mandate that was paused for taking """
                                              """an executive-branch office, or a party change."""))

    date_end = models.DateField(blank=True, null=True,
                                verbose_name=_("Date ended"),
                                help_text=_("""Date in which this mandate ended, paused for taking an """
                                            """executive-branch office, or affiliation change."""))

    party = models.ForeignKey("PoliticalParty",
                              verbose_name=_("Party"),
                              help_text=_("""Party the legislator was affiliated to during this """
                                          """mandate."""),
                              blank=True,
                              null=True)

    state = models.CharField(blank=True, null=True,
                             max_length=512,
                             verbose_name=_("State"),
                             help_text=_("""The state where the legislator comes from."""))

    def __unicode__(self):
        if self.date_end:
            return u"%s's ongoing mandate started on %s, affiliated with %s" % (self.legislator.name,
                                                                                str(self.date_start),
                                                                                self.party)

        return u"%s's mandate started on %s ended on %s, affiliated with %s" % (self.legislator.name,
                                                                                str(self.date_start),
                                                                                str(self.date_end),
                                                                                self.party)


class ExpenseNature(models.Model):

    class Meta:
        verbose_name = _("Expense nature")
        verbose_name_plural = _("Expense natures")

    original_id = models.CharField(blank=True, null=True,
                                   max_length=512,
                                   verbose_name=_("Original ID"))

    name = models.CharField(max_length=512,
                            verbose_name=_("Expense nature name"))

    def __unicode__(self):
        return u"%s" % self.name


class CollectionRun(models.Model):

    class Meta:
        verbose_name = _("Collection")
        verbose_name = _("Collections")

    date = models.DateField(verbose_name=_("Collection date"))
    legislature = models.ForeignKey("Legislature")
    committed = models.BooleanField(default=False)

    def __unicode__(self):
        return u"Collection run on %s for %s" % (self.date, unicode(self.legislature))


class AbstractExpense(models.Model):

    class Meta:
        abstract = True

    original_id = models.CharField(blank=True, null=True,
                                   max_length=512,
                                   verbose_name=_("Original ID"))

    number = models.CharField(max_length=512,
                              verbose_name=_("Document number"),
                              help_text=_("Usually a receipt ID."))

    nature = models.ForeignKey('ExpenseNature')

    date = models.DateField(verbose_name=_("Expense date"))

    value = models.DecimalField(max_digits=10,
                                decimal_places=2,
                                verbose_name=_("Value"),
                                help_text=_("The total amount spent."),
                                blank=True,
                                null=True)

    expensed = models.DecimalField(max_digits=10, decimal_places=2,
                                   verbose_name=_("Expensed"),
                                   help_text=_("Amount reimbursed to the legislator."))

    mandate = models.ForeignKey("Mandate")

    supplier = models.ForeignKey("Supplier")

    def __unicode__(self):
        return u"%s expensed by %s on %s (document %s)" % (str(self.expensed),
                                                           self.mandate.legislator.name,
                                                           str(self.date),
                                                           self.number)


class Expense(AbstractExpense):

    class Meta:
        verbose_name = _("Expense")
        verbose_name_plural = _("Expenses")

    def save(self, *args, **kwargs):
        if settings.expense_locked_for_collection:
            raise RuntimeError("You should not touch Expense while collecting, use ArchivedExpense instead.")
        super(Expense, self).save(*args, **kwargs)


class ArchivedExpense(AbstractExpense):

    class Meta:
        verbose_name = _("Archived expense")
        verbose_name_plural = _("Archived expenses")

    collection_run = models.ForeignKey("CollectionRun")


class Supplier(models.Model):

    class Meta:
        verbose_name = _("Supplier")
        verbose_name_plural = _("Suppliers")

    name = models.CharField(verbose_name=_("Name"), max_length=200)

    identifier = models.CharField(
        max_length=256,
        verbose_name=_("Supplier identifier"))

    date_opened = models.DateField(
        verbose_name=_("Date Opened"),
        blank=True,
        null=True)

    trade_name = models.CharField(
        verbose_name=_("Trade Name"),
        max_length=200,
        blank=True,
        null=True)

    address = models.TextField(
        verbose_name=_("Address"),
        blank=True,
        null=True)

    juridical_nature = models.CharField(
        verbose_name=_("Juridical Nature"),
        max_length=200,
        blank=True,
        null=True)

    status = models.NullBooleanField(
        verbose_name=_("Status"),
        blank=True,
        null=True)

    main_economic_activity = models.CharField(
        verbose_name=_("Main Economic Activity"),
        max_length=200,
        blank=True,
        null=True)

    last_change = models.DateTimeField(
        verbose_name=_('Last Change'),
        auto_now=True,
        blank=True,
        null=True)

    @property
    def identifier_with_mask(self):
        try:
            if len(self.identifier) == 11:
                return Cpf().format(self.identifier)
            elif len(self.identifier) == 14:
                return Cnpj().format(self.identifier)
            else:
                return self.identifier
        except:
            return self.identifier

    @property
    def identifier_label(self):
        if len(self.identifier) == 11:
            return 'CPF'
        elif len(self.identifier) == 14:
            return 'CNPJ'
        else:
            return _('Identifier')

    def __unicode__(self):
        return u"%s (%s)" % (self.name, self.identifier_with_mask)


# Consolidated data models
class PerNature(models.Model):
    institution = models.ForeignKey("Institution")
    legislature = models.ForeignKey("Legislature",
                                    blank=True,
                                    null=True)
    date_start = models.DateField()
    date_end = models.DateField()
    nature = models.ForeignKey("ExpenseNature")
    expensed = models.DecimalField(max_digits=20, decimal_places=2)


class PerNatureByYear(models.Model):
    institution = models.ForeignKey("Institution")
    year = models.IntegerField()
    nature = models.ForeignKey("ExpenseNature")
    expensed = models.DecimalField(max_digits=20, decimal_places=2)


class PerNatureByMonth(models.Model):
    institution = models.ForeignKey("Institution")
    date = models.DateField()
    nature = models.ForeignKey("ExpenseNature")
    expensed = models.DecimalField(max_digits=20, decimal_places=2)


class PerLegislator(models.Model):
    institution = models.ForeignKey("Institution")
    legislature = models.ForeignKey("Legislature",
                                    blank=True,
                                    null=True)
    legislator = models.ForeignKey("Legislator")
    date_start = models.DateField()
    date_end = models.DateField()
    expensed = models.DecimalField(max_digits=20, decimal_places=2)


class BiggestSupplierForYear(models.Model):
    supplier = models.ForeignKey("Supplier")
    year = models.IntegerField()
    expensed = models.DecimalField(max_digits=20, decimal_places=2)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Estêvão Samuel Procópio
# Copyright (©) 2010-2013 Gustavo Noronha Silva
# Copyright (©) 2013 Marcelo Jorge Vieira
# Copyright (©) 2014 Wilson Pinto Júnior
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import os

from cStringIO import StringIO

from basecollector import BaseCollector
from django.core.files import File
from datetime import datetime

from montanha.models import *


class ALGO(BaseCollector):
    TITLE_REGEX = re.compile(r'\d+ - (.*)')

    def __init__(self, *args, **kwargs):
        super(ALGO, self).__init__(*args, **kwargs)

        self.base_url = 'http://al.go.leg.br'

        try:
            institution = Institution.objects.get(siglum='ALGO')
        except Institution.DoesNotExist:
            institution = Institution(siglum='ALGO', name=u'Assembléia Legislativa do Estado de Goiás')
            institution.save()

        try:
            self.legislature = Legislature.objects.all().filter(institution=institution).order_by('-date_start')[0]
        except IndexError:
            self.legislature = Legislature(institution=institution,
                                           date_start=datetime(2011, 1, 1),
                                           date_end=datetime(2014, 12, 31))
            self.legislature.save()

        self.expenses_nature_cached = {}

    def _normalize_party_siglum(self, siglum):
        names_map = {
            'SDD': 'Solidariedade',
        }
        return names_map.get(siglum, siglum)

    def update_legislators(self):
        headers = {
            'Referer': self.base_url + '/deputado/',
            'Origin': self.base_url,
        }
        data = self.retrieve_uri(self.base_url + '/deputado/', headers=headers)

        url_regex = re.compile(r'.*id/(\d+)')
        email_regex = re.compile(r'Email: (.*)')

        for tr in data.find(id='gNome').find('tbody').findAll('tr'):
            tds = tr.findAll('td')

            entry = {}

            entry['url'] = tds[0].find('a')['href']
            entry['id'] = url_regex.match(entry['url']).group(1)
            entry['nome'] = tds[0].find('a').text
            entry['party'] = tds[1].text
            entry['telefone'] = tds[2].text
            entry['fax'] = tds[3].text
            entry['email'] = tds[4].find('img').get('title')

            if entry['email']:
                entry['email'] = email_regex.match(entry['email']).group(1).strip()

            party_siglum = self._normalize_party_siglum(entry["party"])
            party, party_created = PoliticalParty.objects.get_or_create(
                siglum=party_siglum
            )

            self.debug("New party: %s" % unicode(party))

            legislator, created = Legislator.objects.get_or_create(name=entry['nome'])

            legislator.site = self.base_url + entry['url']
            legislator.email = entry['email']
            legislator.save()

            if created:
                self.debug("New legislator: %s" % unicode(legislator))
            else:
                self.debug("Found existing legislator: %s" % unicode(legislator))

            mandate = self.mandate_for_legislator(legislator, party, original_id=entry["id"])

    def update_data_for_year(self, mandate, year):
        self.debug("Updating data for year %d" % year)
        for month in range(1, 13):
            self.update_data_for_month(mandate, year, month)

    @classmethod
    def parse_title(self, title):
        if '-' in title:
            match = self.TITLE_REGEX.search(title)

            if match:
                return match.group(1).encode('utf-8')

        return title.encode('utf-8')

    MONEY_RE = re.compile(r'([0-9.,]+)[,.]([0-9]{2})$')

    @classmethod
    def parse_money(self, value):
        match = self.MONEY_RE.search(value)

        if match:
            return float('%s.%s' % (match.group(1).replace('.', '').replace(',', ''), match.group(2)))
        else:
            raise ValueError('Cannot convert %s to float (money)' % value)

    def find_data_for_month(self, mandate, year, month):
        url = '%s/transparencia/verbaindenizatoria/exibir?ano=%d&mes=%d&parlamentar_id=%s' % (
            self.base_url, year, month, mandate.original_id)
        data = self.retrieve_uri(url, force_encoding='utf8')

        if u'parlamentar não prestou contas para o mês' in data.text:
            self.debug("not found data for: %s -> %d/%d" % (
                unicode(mandate.legislator), year, month))
            raise StopIteration

        container = data.find('div', id='verba')

        if not container:
            self.debug('div#verba not found')

        table = container.find('table', recursive=False)

        if not table:
            self.debug("table.tabela-verba-indenizatoria not found")
            raise StopIteration

        group_trs = table.findAll('tr', {'class': 'verba_titulo'})

        for tr in group_trs:
            budget_title = self.parse_title(tr.text)
            budget_subtitle = None

            while True:
                tr = tr.findNext('tr')

                if not tr:
                    break

                tr_class = tr.get('class')

                if tr.get('class') == 'verba_titulo':
                    break

                elif tr_class == 'info-detalhe-verba':
                    for detail_tr in tr.find('tbody').findAll('tr'):
                        tds = detail_tr.findAll('td')

                        data = {
                            'budget_title': budget_title,
                            'budget_subtitle': budget_subtitle
                        }

                        data['nome'] = tds[0].text
                        data['cpf_cnpj'] = self.normalize_cnpj_or_cpf(tds[1].text)
                        data['date'] = tds[2].text
                        data['number'] = tds[3].text
                        data['value_presented'] = self.parse_money(tds[4].text)
                        data['value_expensed'] = self.parse_money(tds[5].text)

                        self.debug(u'Generated JSON: %s' % data)

                        yield data

                elif tr_class == 'subtotal':
                    continue

                elif len(tr.findAll('td')) == 3:
                    tds = tr.findAll('td')
                    budget_subtitle = self.parse_title(tds[0].text)

                    next_tr = tr.findNext('tr')
                    break_classes = ('subtotal', 'info-detalhe-verba', 'verba_titulo')

                    if next_tr.get('class') in break_classes:
                        continue

                    value_presented = self.parse_money(tds[1].text)
                    value_expensed = self.parse_money(tds[2].text)

                    if not value_expensed or not value_presented:
                        continue

                    data = {
                        'budget_title': budget_title,
                        'budget_subtitle': budget_subtitle,
                        'value_presented': value_presented,
                        'date': '1/%d/%d' % (month, year),
                        'value_expensed': value_expensed,
                        'number': 'Sem número'
                    }

                    self.debug(u'Generated JSON: %s' % data)

                    yield data

    def get_or_create_expense_nature(self, name):
        if name not in self.expenses_nature_cached:
            try:
                nature = ExpenseNature.objects.get(name=name)
            except ExpenseNature.DoesNotExist:
                nature = ExpenseNature(name=name)
                nature.save()

            self.expenses_nature_cached[name] = nature

        return self.expenses_nature_cached[name]

    def update_data_for_month(self, mandate, year, month):
        for data in self.find_data_for_month(mandate, year, month):
            nature = self.get_or_create_expense_nature(data['budget_title'] + ': ' + data['budget_subtitle'])

            name = data.get('nome') or 'Sem nome'
            cpf_cnpj = data.get('cpf_cnpj') or 'Sem CPF/CNPJ (%s)' % name

            try:
                supplier = Supplier.objects.get(identifier=cpf_cnpj)
            except Supplier.DoesNotExist:
                supplier = Supplier(identifier=cpf_cnpj, name=name)
                supplier.save()

            date = datetime.strptime(data['date'], '%d/%m/%Y')
            expense = ArchivedExpense(
                number=data['number'],
                nature=nature,
                date=date,
                value=data['value_presented'],
                expensed=data['value_expensed'],
                mandate=mandate,
                supplier=supplier,
                collection_run=self.collection_run)

            expense.save()

    def update_images(self):
        mandates = Mandate.objects.filter(legislature=self.legislature, legislator__picture='')

        headers = {
            'Referer': self.base_url + '/deputado/',
            'Origin': self.base_url,
        }
        deputado_data = self.retrieve_uri(self.base_url + '/deputado/', headers=headers)

        for mandate in mandates:
            leg = mandate.legislator
            found_text = deputado_data.find(text=re.compile(leg.name))

            if not found_text:
                self.debug('Legislator not found in page: %s' % mandate.legislator.name)
                continue

            tr = found_text.findParents('tr')[0]
            tds = tr.findAll('td')

            detail_path = tds[0].find('a')['href']
            detail_url = self.base_url + detail_path
            detail_data = self.retrieve_uri(detail_url, headers=headers)

            photo_container = detail_data.find('div', {'class': re.compile(r'foto')})
            photo_url = photo_container.find('img')['src']
            photo_data = self.retrieve_uri(self.base_url + photo_url, post_process=False, return_content=True)

            photo_buffer = StringIO(photo_data)
            photo_buffer.seek(0)

            leg.picture.save(os.path.basename(photo_url), File(photo_buffer))
            leg.save()

            self.debug('Saved %s Image URL: %s' % (leg.name, photo_url))

        else:
            self.debug('All legislators have photos')



# -*- coding: utf-8 -*-
#
# Copyright (©) 2013 Gustavo Noronha Silva <gustavo@noronha.eti.br>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import locale
from django import template
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

from montanha.cpf import Cpf
from montanha.cnpj import Cnpj


register = template.Library()


@register.filter
def format_currency(value):
    if value:
        return mark_safe(locale.currency(value, grouping=True).replace(" ", "&nbsp;"))
    else:
        return ''


@register.filter
def itercycle(iterable, counter):
    return iterable[counter % len(iterable)]


@register.simple_tag(takes_context=True)
def sortable_th(context, tag_id, is_default=False):
    request = context["request"]

    th_attr = 'id="%s"' % tag_id
    if request.GET.get("order_by") == tag_id or (is_default and "order_by" not in request.GET):
        th_attr += ' class="sortable'
        if request.GET.get("asc"):
            th_attr += ' sorted-asc'
        else:
            th_attr += ' sorted'
        th_attr += '"'

    return mark_safe(th_attr)


@register.simple_tag
def expenses_table(*args):
    return render_to_string('expenses_table.html', dict(columns=args))


@register.simple_tag(takes_context=True)
def expenses_data_table(context, query_name, col_types, item_id=None):
    columns = []
    for c in col_types:
        if c == 'm':
            columns.append("{ sType: 'money' }")
        else:
            columns.append('null')
    c = dict(institution=context['institution'],
             legislature=context['legislature'],
             filter_spec=context['filter_spec'],
             item_id=item_id,
             query_name=query_name, columns=columns)
    return render_to_string('expenses_data_table.html', c)


@register.simple_tag
def biggest_suppliers_table(*args):
    return render_to_string('biggest_suppliers_table.html', dict(columns=args))


@register.simple_tag(takes_context=True)
def biggest_suppliers_data_table(context, query_name, col_types, item_id=None):
    columns = []
    for c in col_types:
        if c == 'm':
            columns.append("{ sType: 'money' }")
        else:
            columns.append('null')
    c = dict(institution=context['institution'],
             legislature=context['legislature'],
             filter_spec=context['filter_spec'],
             item_id=item_id,
             query_name=query_name, columns=columns)
    return render_to_string('biggest_suppliers_data_table.html', c)


# FIXME: duplicated code
@register.filter
def supplier_with_mask(value):
    try:
        if len(value) == 11:
            return mark_safe(Cpf().format(value))
        elif len(value) == 14:
            return mark_safe(Cnpj().format(value))
        else:
            return value
    except:
        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from olhoneles import __version__

tests_require = [
    'nose',
    'coverage',
    'factory_boy',
]

setup(
    name='olhoneles',
    version=__version__,
    description='Activities of the Brazilian legislators',
    long_description='''
A simple software to allow easier watching of the activities of the Brazilian
legislators
''',
    keywords='brazilian legislators web',
    author='OlhoNeles.org',
    author_email='montanha@olhoneles.org',
    url='http://olhoneles.org',
    license='AGPL',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'BeautifulSoup>=3.2.1,<3.3.0',
        'Django>=1.6.1,<1.7.0',
        'Pillow>=2.3.1,<2.4.0',
        'South>=0.8.4,<0.9.0',
        'django-parsley>=0.4,<0.5.0',
        'django-bootstrap-toolkit==2.14',
        'easy-thumbnails>=2.2.0,<2.3.0',
        'requests>=2.2.1,<2.3.0',
        'chardet>=2.2.1,<2.3.0',
        'pandas>=0.13.1,<0.14.0',
        'derpconf>=0.4.9,<0.5.0',
        'django-recaptcha>=0.0.9,<0.1.0',
        'raven>=5.1.1,<5.2.0',
    ],
    dependency_links = [
        'https://github.com/gnoronha/django-bootstrap-toolkit/archive/0f0ff43eeab8e19ee8d8021460f1a4abf8303bde.zip#egg=django-bootstrap-toolkit-2.14'
    ],
    extras_require={
        'tests': tests_require,
    },
)
